@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) Cardiff University (2018-2021)
 # SPDX-License-Identifier: MIT
 
@@ -10,21 +9,21 @@ To search for all available datasets:
 >>> from gwosc import datasets
 >>> datasets.find_datasets()
 ['GW150914', 'GW151226', 'GW170104', 'GW170608', 'GW170814', 'GW170817', 'LVT151012', 'O1', 'S5', 'S6']
->>> datasets.find_datasets(detector='V1')
+>>> datasets.find_datasets(detector="V1")
 ['GW170814', 'GW170817']
->>> datasets.find_datasets(type='run')
+>>> datasets.find_datasets(type="run")
 ['O1', 'S5', 'S6']
 
 To query for the GPS time of an event dataset (or vice-versa):
 
->>> datasets.event_gps('GW170817')
+>>> datasets.event_gps("GW170817")
 1187008882.43
 >>> datasets.event_at_gps(1187008882)
 'GW170817'
 
 Similar queries are available for observing run datasets:
 
->>> datasets.run_segment('O1')
+>>> datasets.run_segment("O1")
 (1126051217, 1137254417)
 >>> datasets.run_at_gps(1135136350)  # event_gps('GW151226')
 'O1'
@@ -44,8 +43,10 @@ from . import (
     urls,
     utils,
 )
+from .api import DEFAULT_URL
+from .api import v2 as apiv2
 
-__author__ = 'Duncan Macleod <duncan.macleod@ligo.org>'
+__author__ = "Duncan Macleod <duncan.macleod@ligo.org>"
 
 _IGNORE = {
     "tenyear",
@@ -55,39 +56,51 @@ _IGNORE = {
 
 
 def _match_dataset(targetdetector, detectors, targetsegment, segment):
-    """Returns `True` if the dataset metadata matches the target
-    """
+    """Returns `True` if the dataset metadata matches the target"""
+    warnings.warn(
+        "_match_dataset function will be removed in future versions.",
+        DeprecationWarning,
+        stacklevel=2,
+    )
     if targetdetector not in set(detectors) | {None}:
         return False
-    if targetsegment is None or utils.segments_overlap(
-            targetsegment, segment):
+    if targetsegment is None or utils.segments_overlap(targetsegment, segment):
         return True
 
 
-def _run_datasets(detector=None, segment=None, host=api.DEFAULT_URL):
-    meta = api.fetch_dataset_json(0, api._MAX_GPS, host=host)["runs"]
-    for epoch, metadata in meta.items():
-        # ignore tenyear, etc...
-        if epoch in _IGNORE:
+def _run_datasets(detector=None, segment=None, host=DEFAULT_URL):
+    runs = apiv2.fetch_runs(host=host)
+    for run in runs:
+        # Skip if detector was given and it doesn't match
+        if detector is not None and detector not in run["detectors"]:
             continue
-        rundets = set(metadata["detectors"])
-        runseg = (metadata['GPSstart'], metadata['GPSend'])
-        if _match_dataset(detector, rundets, segment, runseg):
-            yield epoch
+        # Skip if segment was given and it doesn't overlap
+        if segment is not None:
+            run_segment = (run["gps_start"], run["gps_end"])
+            do_overlap = utils.segments_overlap(run_segment, segment)
+            if not do_overlap:
+                continue
+        yield run["name"]
 
 
-def _catalog_datasets(host=api.DEFAULT_URL):
-    yield from api.fetch_cataloglist_json(host=host).keys()
+def _catalog_datasets(host=DEFAULT_URL):
+    for catalog in apiv2.fetch_catalogs(host=host):
+        yield catalog["name"]
 
 
 def _match_event_dataset(
-        dataset,
-        catalog=None,
-        version=None,
-        detector=None,
-        segment=None,
-        host=api.DEFAULT_URL,
+    dataset,
+    catalog=None,
+    version=None,
+    detector=None,
+    segment=None,
+    host=DEFAULT_URL,
 ):
+    warnings.warn(
+        "_match_event_dataset function will be removed in future versions.",
+        DeprecationWarning,
+        stacklevel=2,
+    )
     # get strain file list (matching catalog and version)
     full = bool(detector or segment)
     try:
@@ -123,30 +136,32 @@ def _match_event_dataset(
 
 
 def _event_datasets(
-        detector=None,
-        segment=None,
-        catalog=None,
-        version=None,
-        host=api.DEFAULT_URL,
+    detector=None,
+    segment=None,
+    catalog=None,
+    version=None,
+    host=DEFAULT_URL,
 ):
-    full = detector is not None or segment is not None
-    events = {}
-    for dset, meta in api.fetch_allevents_json(
-        host=host,
-        full=full,
-    )["events"].items():
-        if _match_event_dataset(
-            dset,
-            detector=detector,
-            segment=segment,
-            catalog=catalog,
-            version=version,
-            host=host,
-        ):
-            events[dset] = meta
+    # This segment expansion is needed to account for segments
+    # that do not contain events, but are contained in the
+    # 4096s event dataset (strain file)
+    if segment is not None:
+        start, end = segment
+        expanded_segment = (start, end + 4096)
+    else:
+        expanded_segment = None
 
-    def _rank_catalog(x):
-        cat = x["catalog.shortName"].lower()
+    events = apiv2.fetch_event_versions(
+        segment=expanded_segment, catalogs=catalog, host=host
+    )
+    if version is not None:
+        # Filter by version
+        events = filter(lambda e: e["version"] == version, events)
+    if detector is not None:
+        # Filter by detector
+        events = filter(lambda e: detector in e["detectors"], events)
+
+    def _rank_catalog(cat):
         if "confident" in cat:
             return 1
         for word in ("marginal", "preliminary"):
@@ -154,31 +169,30 @@ def _event_datasets(
                 return 10
         return 5
 
-    def _gps_distance(x):
-        gps = x["GPS"]
-        if not segment:
+    def _gps_distance(gps):
+        if segment is None:
             return 0
         return int(abs(segment[0] - gps))
 
-    for dset, _ in sorted(
-        events.items(),
-        key=lambda x: (
-            _gps_distance(x[1]),
-            _rank_catalog(x[1]),
-            -x[1]["version"],
+    for event in sorted(
+        events,
+        key=lambda e: (
+            _gps_distance(e["gps"]),
+            _rank_catalog(e["catalog"].lower()),
+            -e["version"],
         ),
     ):
-        yield dset
+        yield f"{event['shortName']}"
 
 
 def _iter_datasets(
-        detector=None,
-        type=None,
-        segment=None,
-        catalog=None,
-        version=None,
-        match=None,
-        host=api.DEFAULT_URL,
+    detector=None,
+    type=None,
+    segment=None,
+    catalog=None,
+    version=None,
+    match=None,
+    host=DEFAULT_URL,
 ):
     # get queries
     type = str(type).rstrip("s").lower()
@@ -191,8 +205,8 @@ def _iter_datasets(
         for key, value in dict(catalog=catalog, version=version).items():
             if value is not None:
                 warnings.warn(
-                    "the '{}' keyword is only relevant when querying "
-                    "for event datasets, it will be ignored now".format(key),
+                    f"the '{key}' keyword is only relevant when querying "
+                    "for event datasets, it will be ignored now",
                 )
 
     if match:
@@ -205,21 +219,30 @@ def _iter_datasets(
 
     # search for events and datasets
     for needthis, collection in (
-        (needruns, _run_datasets(
-            detector=detector,
-            host=host,
-            segment=segment,
-        )),
-        (needcatalogs, _catalog_datasets(
-            host=host,
-        )),
-        (needevents, _event_datasets(
-            detector=detector,
-            segment=segment,
-            host=host,
-            version=version,
-            catalog=catalog,
-        )),
+        (
+            needruns,
+            _run_datasets(
+                detector=detector,
+                host=host,
+                segment=segment,
+            ),
+        ),
+        (
+            needcatalogs,
+            _catalog_datasets(
+                host=host,
+            ),
+        ),
+        (
+            needevents,
+            _event_datasets(
+                detector=detector,
+                segment=segment,
+                host=host,
+                version=version,
+                catalog=catalog,
+            ),
+        ),
     ):
         if not needthis:
             continue
@@ -227,13 +250,13 @@ def _iter_datasets(
 
 
 def find_datasets(
-        detector=None,
-        type=None,
-        segment=None,
-        match=None,
-        catalog=None,
-        version=None,
-        host=api.DEFAULT_URL,
+    detector=None,
+    type=None,
+    segment=None,
+    match=None,
+    catalog=None,
+    version=None,
+    host=DEFAULT_URL,
 ):
     """Find datasets available on the given GW open science host
 
@@ -256,7 +279,7 @@ def find_datasets(
 
     host : `str`, optional
         the URL of the GWOSC host to query, defaults to
-        https://www.gw-openscience.org
+        https://gwosc.org
 
     Returns
     -------
@@ -271,42 +294,54 @@ def find_datasets(
     >>> find_datasets()
     ['GW150914', 'GW151226', 'GW170104', 'GW170608', 'GW170814', 'GW170817',
      'LVT151012', 'O1', 'S5', 'S6']
-    >>> find_datasets(detector='V1')
+    >>> find_datasets(detector="V1")
     ['GW170814', 'GW170817']
-    >>> find_datasets(type='event')
+    >>> find_datasets(type="event")
     ['GW150914', 'GW151226', 'GW170104', 'GW170608', 'GW170814', 'GW170817',
      'LVT151012']
     """
-    return sorted(list(_iter_datasets(
-        detector=detector,
-        type=type,
-        segment=segment,
-        catalog=catalog,
-        version=version,
-        match=match,
-        host=host,
-    )))
+    return sorted(
+        list(
+            _iter_datasets(
+                detector=detector,
+                type=type,
+                segment=segment,
+                catalog=catalog,
+                version=version,
+                match=match,
+                host=host,
+            )
+        )
+    )
 
 
 # -- event utilities ----------------------------------------------------------
 
+
 def _event_metadata(
-        event,
-        catalog=None,
-        version=None,
-        full=True,
-        host=api.DEFAULT_URL,
+    event,
+    catalog=None,
+    version=None,
+    full=True,
+    host=DEFAULT_URL,
 ):
-    return list(api._fetch_allevents_event_json(
-        event,
-        catalog=catalog,
-        version=version,
-        full=full,
-        host=host,
-    )["events"].values())[0]
+    warnings.warn(
+        "_event_metadata function will be removed in future versions.",
+        DeprecationWarning,
+        stacklevel=2,
+    )
+    return list(
+        api._fetch_allevents_event_json(
+            event,
+            catalog=catalog,
+            version=version,
+            full=full,
+            host=host,
+        )["events"].values()
+    )[0]
 
 
-def event_gps(event, catalog=None, version=None, host=api.DEFAULT_URL):
+def event_gps(event, catalog=None, version=None, host=DEFAULT_URL):
     """Returns the GPS time of an open-data event
 
     Parameters
@@ -323,7 +358,7 @@ def event_gps(event, catalog=None, version=None, host=api.DEFAULT_URL):
 
     host : `str`, optional
         the URL of the GWOSC host to query, defaults to
-        https://www.gw-openscience.org
+        https://gwosc.org
 
     Returns
     -------
@@ -333,26 +368,23 @@ def event_gps(event, catalog=None, version=None, host=api.DEFAULT_URL):
     Examples
     --------
     >>> from gwosc.datasets import event_gps
-    >>> event_gps('GW170817')
+    >>> event_gps("GW170817")
     1187008882.43
-    >>> event_gps('GW123456')
+    >>> event_gps("GW123456")
     ValueError: no event dataset found for 'GW123456'
     """
-    return _event_metadata(
-        event,
-        catalog=catalog,
-        version=version,
-        full=False,
-        host=host,
-    )['GPS']
+    event = apiv2.fetch_event_version(
+        event, catalog=catalog, version=version, host=host
+    )
+    return event["gps"]
 
 
 def event_segment(
-        event,
-        detector=None,
-        catalog=None,
-        version=None,
-        host=api.DEFAULT_URL,
+    event,
+    detector=None,
+    catalog=None,
+    version=None,
+    host=DEFAULT_URL,
 ):
     """Returns the GPS ``[start, stop)`` interval covered by an event dataset
 
@@ -373,12 +405,12 @@ def event_segment(
 
     host : `str`, optional
         the URL of the GWOSC host to query, defaults to
-        https://www.gw-openscience.org
+        https://gwosc.org
 
     Returns
     -------
     start, end : `int`
-        the GPS ``[start, end)`` interval covered by this run dataset
+        the GPS ``[start, end)`` interval covered by this event dataset
 
     Examples
     --------
@@ -386,25 +418,28 @@ def event_segment(
     >>> event_segment("GW150914")
     segment(1126257415, 1126261511)
     """
-    data = _event_metadata(
-        event,
-        catalog=catalog,
-        version=version,
-        full=True,
-        host=host,
+
+    strain_files = apiv2.fetch_event_strain_data(
+        event, detector=detector, version=None, catalog=None, host=host
     )
 
-    if not data["strain"]:  # pragma: no cover
+    if not strain_files:  # pragma: no cover
         raise ValueError(
-            "event '{}' has no strain files".format(event),
+            f"event '{event}' has no strain files",
         )
 
-    return utils.strain_extent(
-        urls.sieve(data["strain"], detector=detector),
+    # Return the strain files extent (mod of utils.strain_extent)
+    starts, ends = zip(
+        *[
+            (afile["gps_start"], afile["gps_start"] + afile["duration"])
+            for afile in strain_files
+        ]
     )
 
+    return min(starts), max(ends)
 
-def event_at_gps(gps, host=api.DEFAULT_URL, tol=1):
+
+def event_at_gps(gps, host=DEFAULT_URL, tol=1):
     """Returns the name of the open-data event matching the GPS time
 
     This function will return the first event for which
@@ -417,7 +452,7 @@ def event_at_gps(gps, host=api.DEFAULT_URL, tol=1):
 
     host : `str`, optional
         the URL of the GWOSC host to query, defaults to
-        https://www.gw-openscience.org
+        https://gwosc.org
 
     tol : `float`, optional
         the search window (in seconds), default: ``1``
@@ -437,18 +472,16 @@ def event_at_gps(gps, host=api.DEFAULT_URL, tol=1):
     >>> from gwosc.datasets import event_at_gps
     >>> event_at_gps(1187008882)
     'GW170817'
-    >>> event_at_gps(1187008882, tol=.1)
+    >>> event_at_gps(1187008882, tol=0.1)
     ValueError: no event found within 0.1 seconds of 1187008882
     """
-    for meta in api.fetch_allevents_json(host=host)["events"].values():
-        egps = meta['GPS']
-        if abs(egps - gps) <= tol:
-            return meta["commonName"]
-    raise ValueError("no event found within {0} seconds of {1}".format(
-        tol, gps))
+    events = list(apiv2.fetch_event_versions(segment=(gps - tol, gps + tol)))
+    if len(events) < 1:
+        raise ValueError(f"no event found within {tol} seconds of {gps}")
+    return events[0]["name"]
 
 
-def event_detectors(event, catalog=None, version=None, host=api.DEFAULT_URL):
+def event_detectors(event, catalog=None, version=None, host=DEFAULT_URL):
     """Returns the `set` of detectors that observed an event
 
     Parameters
@@ -456,14 +489,17 @@ def event_detectors(event, catalog=None, version=None, host=api.DEFAULT_URL):
     event : `str`
         the name of the event to query
 
-    host : `str`, optional
-        the URL of the GWOSC host to query, defaults to
-        https://www.gw-openscience.org
-
     version : `int`, `None`, optional
         the version of the data release to use,
         defaults to the highest available version
 
+    catalog : `str`, optional
+        name of catalogue that hosts this event
+
+    host : `str`, optional
+        the URL of the GWOSC host to query, defaults to
+        https://gwosc.org
+
     Returns
     -------
     detectors : `set`
@@ -476,19 +512,16 @@ def event_detectors(event, catalog=None, version=None, host=api.DEFAULT_URL):
     >>> event_detectors("GW150914")
     {'H1', 'L1'}
     """
-    data = _event_metadata(
-        event,
-        catalog=catalog,
-        version=version,
-        full=True,
-        host=host,
+    event = apiv2.fetch_event_version(
+        event, catalog=catalog, version=version, host=host
     )
-    return set(u["detector"] for u in data["strain"])
+    return set(event["detectors"])
 
 
 # -- run utilities ------------------------------------------------------------
 
-def run_segment(run, host=api.DEFAULT_URL):
+
+def run_segment(run, host=DEFAULT_URL):
     """Returns the GPS ``[start, stop)`` interval covered by a run dataset
 
     Parameters
@@ -498,7 +531,7 @@ def run_segment(run, host=api.DEFAULT_URL):
 
     host : `str`, optional
         the URL of the GWOSC host to query, defaults to
-        https://www.gw-openscience.org
+        https://gwosc.org
 
     Returns
     -------
@@ -508,19 +541,16 @@ def run_segment(run, host=api.DEFAULT_URL):
     Examples
     --------
     >>> from gwosc.datasets import run_segment
-    >>> run_segment('O1')
+    >>> run_segment("O1")
     segment(1126051217, 1137254417)
-    >>> run_segment('Oh dear')
-    ValueError: no run dataset found for 'Oh dear'
+    >>> run_segment("Oh dear")
+    ValueError: Run 'Oh dear' not found.
     """
-    try:
-        meta = api.fetch_dataset_json(0, api._MAX_GPS, host=host)['runs'][run]
-    except KeyError as exc:
-        raise ValueError('no run dataset found for {!r}'.format(exc.args[0]))
-    return meta['GPSstart'], meta['GPSend']
+    run = apiv2.fetch_run(run)
+    return run["gps_start"], run["gps_end"]
 
 
-def run_at_gps(gps, host=api.DEFAULT_URL):
+def run_at_gps(gps, host=DEFAULT_URL):
     """Returns the name of the open-data run dataset matching the GPS time
 
     This function will return the first event for which
@@ -533,7 +563,7 @@ def run_at_gps(gps, host=api.DEFAULT_URL):
 
     host : `str`, optional
         the URL of the GWOSC host to query, defaults to
-        https://www.gw-openscience.org
+        https://gwosc.org
 
     Returns
     -------
@@ -553,17 +583,14 @@ def run_at_gps(gps, host=api.DEFAULT_URL):
     >>> run_at_gps(0)
     ValueError: no run dataset found containing GPS 0
     """
-    for run, meta in api.fetch_dataset_json(
-            0, api._MAX_GPS, host=host)['runs'].items():
-        if run in _IGNORE:
-            continue
-        start, end = meta['GPSstart'], meta['GPSend']
+    for run in apiv2.fetch_runs(host=host):
+        start, end = run["gps_start"], run["gps_end"]
         if start <= gps < end:
-            return run
-    raise ValueError("no run dataset found containing GPS {0}".format(gps))
+            return run["name"]
+    raise ValueError(f"no run dataset found containing GPS {gps}")
 
 
-def dataset_type(dataset, host=api.DEFAULT_URL):
+def dataset_type(dataset, host=DEFAULT_URL):
     """Returns the type of the named dataset
 
     Parameters
@@ -587,18 +614,18 @@ def dataset_type(dataset, host=api.DEFAULT_URL):
     Examples
     --------
     >>> from gwosc.datasets import dataset_type
-    >>> dataset_type('O1')
+    >>> dataset_type("O1")
     'run'
     """
     for type_ in ("run", "catalog", "event"):
         if dataset in find_datasets(type=type_, host=host):
             return type_
     raise ValueError(
-        "failed to determine type for dataset {0!r}".format(dataset),
+        f"failed to determine type for dataset {dataset!r}",
     )
 
 
-def query_events(select, host=api.DEFAULT_URL):
+def query_events(select, host=DEFAULT_URL):
     """Return a list of events filtered by the parameters in `select`
 
     Parameters
@@ -631,26 +658,23 @@ def query_events(select, host=api.DEFAULT_URL):
 
     Parameters:
 
-    - ``gps-time`,
-    - ``mass-1-source``,
-    - ``mass-2-source``,
+    - ``gps-time`` [s],
+    - ``mass-1-source`` [solar mass],
+    - ``mass-2-source`` [solar mass],
     - ``network-matched-filter-snr``,
-    - ``luminosity-distance``,
+    - ``luminosity-distance`` [Mpc],
     - ``chi-eff``,
-    - ``total-mass-source``,
-    - ``chirp-mass``,
-    - ``chirp-mass-source``,
+    - ``total-mass-source`` [solar mass],
+    - ``chirp-mass`` [solar mass],
+    - ``chirp-mass-source`` [solar mass],
     - ``redshift``,
-    - ``far``,
+    - ``far`` [events/year],
     - ``p-astro``,
-    - ``final-mass-source``
+    - ``final-mass-source`` [solar mass],
 
     For a full description of all parameters see
     https://www.gwosc.org/apidocs/#event5
     """
-    return list(
-        api.fetch_filtered_events_json(
-            select=select,
-            host=host,
-        )["events"].keys()
-    )
+    select_query = dict(utils.select_to_query(select, host=host))
+    events = apiv2.fetch_event_versions(select=select_query, host=host)
+    return [f"{e['shortName']}" for e in events]
